"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.render = exports.getView = exports.registerView = exports.viewDictionary = exports.View = void 0;
require("tocca");
const knockout_1 = require("knockout");
const ViewModel_1 = require("./ViewModel");
class View {
    constructor(options, designTimeOptions) {
        this.children = new Array();
        this.tagName = options.tagName;
        this.viewModel = (0, ViewModel_1.extendViewModel)(options.viewModel, designTimeOptions);
        this.optionsChildren = options.children;
        this.children = (0, knockout_1.unwrap)(options.children);
    }
    buildChildren() {
        if (this.element && this.children && this.children.length > 0) {
            const documentFragment = document.createDocumentFragment();
            this.children.forEach((childView) => {
                documentFragment.appendChild(childView.render());
            });
            this.element.appendChild(documentFragment);
        }
    }
    build() {
        (0, ViewModel_1.applyViewModelToElement)(this.viewModel, this.element);
        if (this.children) {
            this.buildChildren();
        }
        if ((0, knockout_1.isObservable)(this.optionsChildren)) {
            this.optionsChildren.subscribe((children) => {
                this.setChildren(...children);
            });
        }
    }
    remove() {
        if (this.element) {
            if (this.element.remove) {
                this.element.remove();
            }
            else {
                if (this.element.parentElement) {
                    this.element.parentElement.removeChild(this.element);
                }
            }
            this.element = null;
        }
    }
    addChild(child) {
        this.children.push(child);
        this.buildChildren();
        return this;
    }
    setChildren(...children) {
        this.empty();
        this.children = children;
        this.buildChildren();
        return this;
    }
    empty() {
        this.children = [];
        if (this.element) {
            this.element.innerHTML = "";
            while (this.element.firstChild) {
                this.element.removeChild(this.element.firstChild);
            }
        }
    }
    render() {
        if (this.element) {
            this.remove();
        }
        if (typeof this.build === "function") {
            this.element = document.createElement(this.tagName);
            this.build();
            return this.element;
        }
        throw "The build method of this UiElement has not been defined";
    }
}
exports.View = View;
exports.viewDictionary = new Map();
function registerView(viewModel, resolver) {
    exports.viewDictionary.set(viewModel.prototype.constructor, resolver);
}
exports.registerView = registerView;
function getView(viewModel) {
    const view = exports.viewDictionary.get(viewModel.constructor);
    if (view.prototype) {
        return new view(viewModel);
    }
    if (typeof view === "function") {
        return view(viewModel);
    }
    throw "View resolver for '" + viewModel.constructor.name + " could not be found";
}
exports.getView = getView;
function render(viewModel) {
    return getView(viewModel).render();
}
exports.render = render;
//# sourceMappingURL=View.js.map