"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.observableArrayProperty = exports.observableProperty = exports.isObservableProperty = void 0;
const knockout_1 = require("knockout");
function isObservableProperty(instance) {
    return instance && !!instance.set;
}
exports.isObservableProperty = isObservableProperty;
function observableProperty(value, setter, notifyAlways = false) {
    let internalProperty;
    if (value === undefined) {
        value = null;
    }
    if ((0, knockout_1.isObservable)(value)) {
        internalProperty = value;
    }
    else {
        internalProperty = (0, knockout_1.observable)(value);
    }
    if (notifyAlways) {
        internalProperty.extend({ notify: "always" });
    }
    const computed = (0, knockout_1.pureComputed)({ read: internalProperty }).extend({ notify: "always" });
    computed.set = (newValue, triggerSetter = true) => {
        if (setter && triggerSetter) {
            setter(newValue, internalProperty);
        }
        else {
            internalProperty(newValue);
        }
    };
    return computed;
}
exports.observableProperty = observableProperty;
function observableArrayProperty(items, setter) {
    let internalProperty;
    if (!items) {
        items = [];
    }
    if ((0, knockout_1.isObservable)(items)) {
        internalProperty = items;
    }
    else {
        internalProperty = (0, knockout_1.observableArray)(items);
    }
    const computed = (0, knockout_1.pureComputed)({ read: internalProperty }).extend({ notify: "always" });
    computed.set = (value, triggerSetter = true) => {
        if (setter && triggerSetter) {
            setter(value, internalProperty);
        }
        else {
            internalProperty(value);
        }
    };
    computed.push = (...itemsToPush) => {
        internalProperty.push(...itemsToPush);
        return itemsToPush.length;
    };
    computed.map = (callBack, thisArg) => {
        return internalProperty().map(callBack, thisArg);
    };
    return computed;
}
exports.observableArrayProperty = observableArrayProperty;
//# sourceMappingURL=observable.js.map